/********************/
/* クラスRunの定義  */
/* （ゲームの実行） */
/********************/
import java.io.*;
import java.awt.*;
import java.awt.event.*;

class Run extends Frame {

	private int count = 0;   // コマを動かした回数
	private int n;   // 行・列の数
	private int ok = 0;   // ゲームの終了
	private int size;   // コマの大きさ
	private Data dt;
	private Run rn = this;
	Insets insets;

	/****************************************/
	/* コンストラクタ                       */
	/*      dt_i : Dataクラスのオブジェクト */
	/****************************************/
	Run(Data dt_i)
	{
					// Frameクラスのコンストラクタの呼び出し
		super("ゲーム");
					// データの設定
		dt   = dt_i;
		size = 50;
		n    = (dt.game == 0) ? 3 : 4;
		setFont(new Font("TimesRoman", Font.BOLD, 20));
					// Windowの大きさの設定
		if (dt.game == 0)
			setSize(3*(size+10)+70, 3*(size+10)+200);
		else
			setSize(4*(size+10)+70, 4*(size+10)+200);
					// ウィンドウを表示
		setVisible(true);
		insets = getInsets();
					// イベントアダプタ
		addWindowListener(new WinEnd());
		addMouseListener(new ClickMouse());
	}

	/******************************/
	/* 上，左，下，右の余白の設定 */
	/******************************/
	public Insets getInsets()
	{
		return new Insets(70, 20, 70, 30);
	}

	/********/
	/* 描画 */
	/********/
	public void paint (Graphics e)
	{
		int i1, i2, x, y;
					// 四角形を描く
		x = insets.left + 20;
		y = insets.top;
		e.setColor(Color.cyan);
		for (i1 = 0; i1 < n; i1++) {
			for (i2 = 0; i2 < n; i2++) {
				e.fill3DRect(x, y, size, size, true);
				x += (size + 10);
			}
			x  = insets.left + 20;
			y += (size + 10);
		}
					// 数字を描く
		x = insets.left + 20;
		y = insets.top;
		e.setColor(Color.black);
		e.setFont(new Font("TimesRoman", Font.BOLD, size));
		for (i1 = 0; i1 < n; i1++) {
			for (i2 = 0; i2 < n; i2++) {
				if (dt.p[i1][i2] > 0) {
					if (dt.p[i1][i2] < 10)
						e.drawString(Integer.toString(dt.p[i1][i2]), x+13, y+size-9);
					else
						e.drawString(Integer.toString(dt.p[i1][i2]), x-2, y+size-9);
				}
				x += (size + 10);
			}
			x  = insets.left + 20;
			y += (size + 10);
		}
					// ゲームの終了
		if (ok > 0) {
			Dimension d = getSize();
			x = insets.left + 20;
			y = d.height - insets.bottom - 25;
			e.setFont(new Font("TimesRoman", Font.PLAIN, 20));
			e.drawString("Congratulation!!", x, y-20);
			e.drawString("Number of Trial : "+Integer.toString(count), x, y);
		}
	}

	/************************************/
	/* マウスがクリックされたときの処理 */
	/************************************/
	class ClickMouse extends MouseAdapter
	{
		public void mouseClicked(MouseEvent e)
		{
			int i1, i2, k1 = -1, k2 = -1, sw, x, xp, y, yp;
							// マウスの位置
			xp = e.getX();
			yp = e.getY();
							// クリックされたコマを決定
			x = insets.left + 20;
			y = insets.top;
			for (i1 = 0; i1 < n && k1 < 0; i1++) {
				for (i2 = 0; i2 < n && k1 < 0; i2++) {
					if (xp > x && xp < x+size && yp > y && yp < y+size) {
						k1 = i1;
						k2 = i2;
					}
					else
						x += (size + 10);
				}
				x  = insets.left + 20;
				y += (size + 10);
			}
							// コマの移動
			sw = 0;

			if (k1 > 0) {                // 上へ移動
				if (dt.p[k1-1][k2] == 0) {
					dt.p[k1-1][k2] = dt.p[k1][k2];
					dt.p[k1][k2]   = 0;
					sw             = 1;
				}
			}

			if (sw == 0 && k1 < n-1) {   // 下へ移動
				if (dt.p[k1+1][k2] == 0) {
					dt.p[k1+1][k2] = dt.p[k1][k2];
					dt.p[k1][k2]   = 0;
					sw             = 1;
				}
			}

			if (sw == 0 && k2 > 0) {     // 左へ移動
				if (dt.p[k1][k2-1] == 0) {
					dt.p[k1][k2-1] = dt.p[k1][k2];
					dt.p[k1][k2]   = 0;
					sw             = 1;
				}
			}

			if (sw == 0 && k2 < n-1) {   // 右へ移動
				if (dt.p[k1][k2+1] == 0) {
					dt.p[k1][k2+1] = dt.p[k1][k2];
					dt.p[k1][k2]   = 0;
					sw             = 1;
				}
			}
							// ゴールに達したか否かのチェックと描画
			if (sw > 0) {
				count++;
				ok = 1;
				for (i1 = 0; i1 < n && ok > 0; i1++) {
					for (i2 = 0; i2 < n && ok > 0; i2++) {
						if (dt.g[i1][i2] != dt.p[i1][i2])
							ok = 0;
					}
				}
				repaint();
			}
		}
	}

	/************/
	/* 終了処理 */
	/************/
	class WinEnd extends WindowAdapter
	{
		public void windowClosing(WindowEvent e) {
			rn.setVisible(false);
		}
	}
}
